#ifndef CONTROLLERTAB_H
#define CONTROLLERTAB_H

#include <QDialog>
#include <QtGui>

#include "motortab.h"


#define NUM_CONTROLLER_LOOPS 				4
#define DI 0
#define PA 1
#define FB 2
#define FF 3

#define NUM_CONTROLLER_PARAM		4
#define KP 	0
#define KI 	1
#define KD 	2
#define WUP 3
#define EP 	4 //Not used 
		
const QString controllersName[NUM_CONTROLLER_LOOPS] = {"Direct", "Parallel", "FeedBack", "FeedForward"};
/*const QString controllerParamsNamesGlobal[NUM_CONTROLLER_PARAM] = {"Kp", "Ki", "Kd", "Wup", "Ep"};*/
const QString controllerParamsNamesGlobal[NUM_CONTROLLER_PARAM] = {"Kp", "Ki", "Kd", "Wup"}; //Epsilon not used

class QTabWidget;

class ControllerMotorTab : public QWidget
{
	Q_OBJECT

	public:
		ControllerMotorTab ( void );
		
		QDoubleSpinBox 	*controllerParam[NUM_CONTROLLER_LOOPS][NUM_CONTROLLER_PARAM];
		QSpinBox				*Tsampling;
		QComboBox				*controllerType;
		
		QComboBox							*controlVar;
		QComboBox							*referenceType;
		QDoubleSpinBox				*referenceValue1;
		QDoubleSpinBox				*referenceValue2;
		QDoubleSpinBox				*referenceValue3;
		QDoubleSpinBox				*referenceValue4;
	
	private:
		void createController	( unsigned int num_controller );
		
		QGroupBox *controllerGroup[NUM_CONTROLLER_LOOPS];
		QString controllerParamsNames[NUM_CONTROLLER_PARAM];

	private slots:
		void handleReferenceTypeChanged(int index);	
};

class ControllerTab : public QWidget
{
	Q_OBJECT

	public:
		ControllerTab ( MotorTab *motor_tab );

		float 				getControllerParamValue 	( unsigned char u_motor , unsigned char u_controller, 
																							unsigned char u_param);
		unsigned char getNumControllers 				( void );
		unsigned char getNumParams 							( void );
		char* 				getControllerName 				( unsigned char u_controller );
		char* 				getControllerParamName 		( unsigned char u_controller );
		unsigned char getControllerType 				( unsigned char u_motor );
		unsigned int 	getControllerSamplingFreq ( unsigned char u_motor );

		unsigned char	getControllerControlVar 	( unsigned char u_motor );
		unsigned char	getControllerRefType			( unsigned char u_motor );
		float 				getControllerRefValue1		( unsigned char u_motor );
		float 				getControllerRefValue2		( unsigned char u_motor );
		float 				getControllerRefValue3		( unsigned char u_motor );
		float 				getControllerRefValue4		( unsigned char u_motor );
	
		void					enableController					( unsigned char u_motor );
		void 					disableController 				( unsigned char u_motor );

		void 					setControllerSamplingFreq	( unsigned char u_motor, unsigned int u_freq );
		void 					setControllerType					( unsigned char u_motor, unsigned char u_type );
		void					setControllerControlVar 	( unsigned char u_motor, unsigned char u_control_var );
		void 					setControllerRefType			( unsigned char u_motor, unsigned char u_ref_type );
		void  				setControllerRefValue1		( unsigned char u_motor, float f_ref_value );
		void	 				setControllerRefValue2		( unsigned char u_motor, float f_ref_value );
		void	 				setControllerRefValue3		( unsigned char u_motor, float f_ref_value );
		void	 				setControllerRefValue4		( unsigned char u_motor, float f_ref_value );
		void  				setControllerParamValue 	( unsigned char u_motor , unsigned char u_controller, 
																							unsigned char u_param, float f_value);

		ControllerMotorTab	*controllerMotorTab[NUM_MOTORS];
};
#endif//CONTROLLERTAB_H
