/***************************************************************************
**                                                                        **
**  QCustomPlot, an easy to use, modern plotting widget for Qt            **
**  Copyright (C) 2011, 2012, 2013 Emanuel Eichhammer                     **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program.  If not, see http://www.gnu.org/licenses/.   **
**                                                                        **
****************************************************************************
**           Author: Emanuel Eichhammer                                   **
**  Website/Contact: http://www.qcustomplot.com/                          **
**             Date: 05.09.13                                             **
**          Version: 1.0.1                                                **
****************************************************************************/

#ifndef PLOTWINDOW_H
#define PLOTWINDOW_H

#include <QMainWindow>
#include "qcustomplot.h"
#include "motortab.h"
#include <string.h>
#include <fstream>

using namespace std;
namespace Ui {
class PlotWindow;
}

#define VARNAMES      13
#define POS           0
#define SPEED         1
#define CURRENT       2
#define REF           3
#define ERROR         4
#define ERROR_REF_FB  5              
#define UFF           6
#define UDI           7 
#define UFB           8
#define UPA           9
#define UALL          10 
#define USAT          11 
#define Y		         	12 

class PlotWindow : public QMainWindow
{
  Q_OBJECT
  

public:
  explicit PlotWindow(QWidget *parent = 0);
  ~PlotWindow();
 
 	void addValue (unsigned char u_motor, unsigned char var, float f_time, float f_value );
	QCustomPlot *globalCustomPlot;

	void savePdf (QString filename);

	double newXCoordA; 
	double newXCoordB; 
	double newYCoordA; 
	double newYCoordB; 

	double defaultXCoordA; 
	double defaultXCoordB; 
	double defaultYCoordA; 
	double defaultYCoordB; 

	void keyPressEvent( QKeyEvent * event );
	void loadPlotsFromSocket(string s_path);
	void setPrintNames (unsigned int var);

	public slots:
		void setupPlot();
		void setTimeLimit ( float f_time );
		void clearData();
		void setCommVarFlag( unsigned char var, unsigned char value);

		void clickedGraph(QMouseEvent *mouseEvent); 
		void mouseReleaseEvent ( QMouseEvent * event );
private slots:
	/*void horzScrollBarChanged(int value);*/
	/*void vertScrollBarChanged(int value);*/
	/*void xAxisChanged(QCPRange range);*/
	/*void yAxisChanged(QCPRange range);*/

  
private:
  Ui::PlotWindow *ui;
	
	QVector<double> motorTime[VARNAMES];
	QVector<double> motorValue[VARNAMES];

	const char 		*printNameVar[VARNAMES];
	const char 		*fileNameVar[VARNAMES];
	unsigned char commVarFlag[VARNAMES];
	float 				fPlotMaxTime;
	unsigned char uPlotFlag;
};

#endif // MAINWINDOW_H
