#ifndef _QT_SECOSOCKET_H_
#define _QT_SECOSOCKET_H_

using namespace std;
#include <QObject>
#include <QtCore/QCoreApplication>

#include <QTcpSocket>
#include <QHostAddress>
#include <cstdio>

#include <time.h>
#include <sys/time.h>
#include <unistd.h>
/*#include "qcustomplot.h"*/

#include "networktab.h"
#include "motortab.h"
#include "controllertab.h"
#include "perturbationtab.h"
#include "communicationtab.h"
#include "communicationdatatab.h"

#include "qledindicator.h"
#include "camerawindow.h"

#include "plotwindow.h"
#include <fstream>
#include <string.h>

#define MAXLINE 10000 /*max text line length*/

/* Motors */
#define MOTOR1					0
#define MOTOR2					1
#define MOTOR3					2
#define MOTOR4					3

/* frame types */
#define MOTOR_TYPE				0
#define CONTROLLER_TYPE		1
#define COMM_TYPE					2
#define DATA_TYPE					3
#define RESET_TYPE				4

/* Motor functs */
#define INIT_MOTOR			0
#define SET_DUTY				1
#define INIT_CURRENT		2

/* Controller functs */
#define	INIT_CONTROLLER				0
#define SET_REF								1
#define START_CONTROLLER			2
#define SET_MOTOR_CONTROLLER	3
#define SET_CONTROLLER_PARAMS	4
#define SEND_CONTROLLER_PARAMS 5
#define SET_PERTURBATION 6

/* Comm functs */
#define SET_COMM_PARAMS		0
#define INIT_SEND_COMM		1
#define COMM_CONFIG				2
#define SEND_COMM_PARAMS 	3

/* Reset functs */
#define RESET_SYSTEM_RESET 	0
#define RESET_ALIFE         1

/* Define Controller PARAMS */
#define PARAM_FF_KP	0
#define PARAM_FF_TD	1
#define PARAM_FF_TI	2
#define PARAM_FF_EP	3
#define PARAM_FF_WU	4

#define PARAM_D_KP	5
#define PARAM_D_TD	6
#define PARAM_D_TI	7
#define PARAM_D_EP	8
#define PARAM_D_WU	9

#define PARAM_FB_KP	10
#define PARAM_FB_TD	11
#define PARAM_FB_TI	12
#define PARAM_FB_EP	13
#define PARAM_FB_WU	14

#define PARAM_P_KP	15
#define PARAM_P_TD	16
#define PARAM_P_TI	17
#define PARAM_P_EP	18
#define PARAM_P_WU	19

/* Define Reference signals */
/*#define FREQ				0 //Valid for SIN and COS */
/*#define DELTA				1*/
/*#define STEP				2*/
/*#define RAMP 				3*/
/*#define PARABOLA		4*/
/*#define SIN					5*/
/*#define COS					6	*/
/*#define TRAPEZOIDAL	7*/

/* Define comm vars */
#define POS     			0 
#define SPEED   			1 
#define CURRENT 			2 
#define REF 					3 
#define ERROR 				4 
#define ERROR_REF_FB 	5 
#define UFF 					6 
#define UDI 					7 
#define UFB 					8 
#define UPA 					9 
#define U 						10 
#define USAT 					11 
#define Y	 						12 

#define CONTROLLER_TEST 0
#define CONTROLLER_PID 	1

class QTcpSocket;

class QtSecoSocket: public QObject
{
	Q_OBJECT;

	public:
  /*explicit 	QtSecoSocket					( PlotWindow *q_plot, QObject *parent = 0 );*/
		explicit 	QtSecoSocket					( PlotWindow *q_plot, QLineEdit *queue, QLedIndicator *led_1, QLedIndicator *led_2, QLedIndicator *led_3, QLedIndicator *led_4, QPushButton *send_button, QPushButton *run_button, QPushButton *reset_button, QPushButton *disconnect_button, CameraWindow *_camera, QObject *parent = 0 );
		bool			connectToServer				( QString address, unsigned int port, CommunicationDataTab *comm_data_tab );
		void 			disconnectFromServer	( void );
		void 			reset									( void );
		void 			sendData 							( unsigned char frame_type, unsigned char funct, 
																			unsigned char u_motor, float data1, float data2, float data3 );
		void 			replaceCommaByPoint 	(	char* sendline );
		void 			replacePointByComma 	(	char* sendline );
		void 			run ( NetworkTab *network_tab, MotorTab *motor_tab, ControllerTab *controller_tab, PerturbationTab *perturbation_tab, CommunicationTab *communication_tab  );

		CommunicationDataTab *commDataTab;
			
		unsigned char getSocketStatus();
		void 					setDataEnd		(	bool flag);
		bool					getDataEnd		( void );
	signals:

	public slots:
		void on_readyRead();
		void sendAlife();
		void socket_disconnected();
	private:
		QTcpSocket* socket;
		PlotWindow 	*plot;
	
		unsigned char socketOnFlag;
		QTimer *m_TAlife;

		string sFilePath;
		string sFileName;

		bool m_bDataEndFlag;
		
    QLedIndicator         *led1;
    QLedIndicator         *led2;
    QLedIndicator         *led3;
    QLedIndicator         *led4;
    QLineEdit             *queueValue;
    QPushButton           *sendButton;
    QPushButton           *runButton;
    QPushButton           *resetButton;
    QPushButton           *disconnectButton;
    CameraWindow          *camera;

		void parseToFile ( char* buf);
		void parseToPlot ( char* buf);

		void startAlifeTimer	( void );
		void stopAlifeTimer 	( void );
};
#endif

