#include "communicationtab.h"

using namespace std;  

/****************************************
 * **************************************/

CommunicationTab::CommunicationTab ( MotorTab *motor_tab)
{
	//Define subtab environment
	QTabWidget *CommunicationsTab = new QTabWidget;
	
	//Get num motors
	unsigned int uNumMotors = motor_tab->getNumMotors();  

	//Add communications tab
	for ( unsigned int i = 0 ; i < uNumMotors; i++)
	{
		communicationMotorTab[i] = new CommunicationMotorTab; 
		CommunicationsTab->addTab( communicationMotorTab[i],	tr("Motor %1").arg(i+1) );
	}
	
	//Set communications tab depending on motors Status
	for ( unsigned int i = 0 ; i < uNumMotors; i++)
		CommunicationsTab->setTabEnabled(i,motor_tab->getMotorStatus(i));

	//Add main layout
	QVBoxLayout *mainLayout = new QVBoxLayout;
	mainLayout->addWidget(CommunicationsTab);
	setLayout(mainLayout);
}
/****************************************
 * **************************************/

CommunicationMotorTab::CommunicationMotorTab()
{
	//QFileInfo fileInfo(".");
	
	//General font type
	//QFont font;
	//font.setBold(true);

	QGroupBox *generalGroup = new QGroupBox();
	QFormLayout *generalLayout = new QFormLayout;
	
	// Sampling Data
	//QLabel		*TtransmissionLabel = new QLabel(tr("Data Transmission Frequency"));
	//TtransmissionLabel->setFont(font);

	Ttransmission = new QSpinBox;
	Ttransmission->setRange(2, 100);
	Ttransmission->setSingleStep(1);
	Ttransmission->setValue(10);
	Ttransmission->setSuffix(" ms");
	Ttransmission->setFixedWidth(150);

	//QLabel		*TlimitTransmissionLabel = new QLabel(tr("Time Transmission Limit"));
	//TlimitTransmissionLabel->setFont(font);

	TlimitTransmission = new QDoubleSpinBox;
	TlimitTransmission->setRange(0, 200);
	TlimitTransmission->setSingleStep(0.01);
	TlimitTransmission->setValue(2.0);
	TlimitTransmission->setSuffix(" sec");
	TlimitTransmission->setLocale(QLocale(QLocale::English));
	TlimitTransmission->setFixedWidth(150);

	generalLayout->addRow(new QLabel(tr("Tx Period:")), Ttransmission);
	generalLayout->addRow(new QLabel(tr("Time Limit:")), TlimitTransmission);

	generalGroup->setLayout(generalLayout);

	//Create Checkboxes
	for ( int i = 0 ; i < NUM_COMM_PARAMS ; i++) 
	{
		DataCommParam[i] = new QCheckBox(dataCommParamNames[i]);
	}

	//Create first layout
	QVBoxLayout *commParam1Layout = new QVBoxLayout;
	for ( int i = 0 ; i < NUM_COMM_PARAMS / 2 ; i++ )
	{
		commParam1Layout->addWidget(DataCommParam[i]);
	}
	
	QGroupBox *param1Group = new QGroupBox();
	param1Group->setLayout(commParam1Layout);
	
	//Create second layout
	QVBoxLayout *commParam2Layout = new QVBoxLayout;
	for ( int i = NUM_COMM_PARAMS / 2  ; i < NUM_COMM_PARAMS  ; i++ )
	{
		commParam2Layout->addWidget(DataCommParam[i]);
	}
	
	QGroupBox *param2Group = new QGroupBox();
	param2Group->setLayout(commParam2Layout);
	
	//Join layouts
	QHBoxLayout *commParamLayout = new QHBoxLayout;
	commParamLayout->addWidget(param1Group);	
	commParamLayout->addWidget(param2Group);	
	
	QGroupBox *paramGroup = new QGroupBox();
	paramGroup->setLayout(commParamLayout);
	
	//Final Layout
	QVBoxLayout *finalLayout = new QVBoxLayout;
	//finalLayout->addWidget(samplingGroup);
	finalLayout->addWidget(generalGroup);
	finalLayout->addWidget(paramGroup);

	finalLayout->addStretch(3);
	setLayout(finalLayout);
}



/*********************************************
 **********************************************/

unsigned char CommunicationTab::getCommunicationParamValue ( unsigned char u_motor , unsigned char u_param) 
{
	return communicationMotorTab[u_motor]->DataCommParam[u_param]->isChecked();
}

/*********************************************
 **********************************************/

unsigned char CommunicationTab::getNumParams ( void )
{
	return NUM_COMM_PARAMS;
}

/*********************************************
 **********************************************/

char* CommunicationTab::getCommunicationParamName ( unsigned char u_param )
{
	//GetIP
	QString nameString = dataCommParamNamesSimple[u_param];
	QByteArray tmp = nameString.toLatin1();
	char *name = tmp.data();
	return name;
}

/*********************************************
 **********************************************/
unsigned int CommunicationTab::getCommunicationTransmissionFreq ( unsigned char u_motor )
{
	float fTransmissionPeriod = (float) communicationMotorTab[u_motor]->Ttransmission->value(); //in ms
	fTransmissionPeriod /= 1000; //go for secs
	unsigned int uTransmissionFreq = (unsigned int) ( 1.0 / fTransmissionPeriod ); //in Hz

	return uTransmissionFreq;
}
/*********************************************
 **********************************************/
float CommunicationTab::getCommunicationTimeLimit ( unsigned char u_motor )
{
	return communicationMotorTab[u_motor]->TlimitTransmission->value();
	
}
/*********************************************
 **********************************************/
		
void 	CommunicationTab::enableCommunication ( unsigned char u_motor )
{
	//CommunicationsTab->setTabEnabled(u_motor,1);
}

/*********************************************
 **********************************************/
void 	CommunicationTab::disableCommunication ( unsigned char u_motor )
{
	//CommunicationsTab->setTabEnabled(u_motor,0);
}

/*********************************************
 **********************************************/
		
void 	CommunicationTab::setCommunicationTransmissionFreq ( unsigned char u_motor , unsigned int u_freq)
{
	float fTransmissionPeriod = (float) ( 1.0 / (float) u_freq ); // in sec
	fTransmissionPeriod *= 1000; //go for msec
	
	communicationMotorTab[u_motor]->Ttransmission->setValue((unsigned int) fTransmissionPeriod);
}
/*********************************************
 **********************************************/
void  CommunicationTab::setCommunicationTimeLimit ( unsigned char u_motor , float f_time_limit)
{
	communicationMotorTab[u_motor]->TlimitTransmission->setValue(f_time_limit);
}
/*********************************************
 **********************************************/
void 	CommunicationTab::setCommunicationParamValue ( unsigned char u_motor , unsigned char u_param, unsigned char u_value)
{
	if ( u_value == 1 )
		communicationMotorTab[u_motor]->DataCommParam[u_param]->setChecked(true);
	else
		communicationMotorTab[u_motor]->DataCommParam[u_param]->setChecked(false);
		

}
