#include "controllertab.h"


using namespace std;

ControllerTab::ControllerTab ( MotorTab *motor_tab )
{
	
	//Define subtab environment
	QTabWidget *ControllersTab = new QTabWidget;

	//Get num motors
	unsigned int uNumMotors = motor_tab->getNumMotors();

	//Add controllers tab
	for ( unsigned int i = 0 ; i < uNumMotors; i++)
	{
		controllerMotorTab[i] = new ControllerMotorTab; 
		ControllersTab->addTab( controllerMotorTab[i],	tr("Motor %1").arg(i+1) );
	}
	//Set controllers tab depending on motors Status
	for ( unsigned int i = 0 ; i < uNumMotors; i++)
		ControllersTab->setTabEnabled(i,motor_tab->getMotorStatus(i));

	//Add main layout
	QVBoxLayout *mainLayout = new QVBoxLayout;
	mainLayout->addWidget(ControllersTab);
	setLayout(mainLayout);

}


/***********************************
 ************************************/

ControllerMotorTab::ControllerMotorTab ( void )
{
	//QFileInfo fileInfo(".");

	//General font type
	//QFont fontBold;
	//fontBold.setBold(true);

	QGroupBox *generalGroup = new QGroupBox();
	QGridLayout *generalLayout = new QGridLayout;
	
	/* General */
	/* Config Period Sampling */
	Tsampling = new QSpinBox;
	Tsampling->setRange(1, 100);
	Tsampling->setSingleStep(1);
	Tsampling->setValue(5);
	Tsampling->setSuffix(" ms");
	Tsampling->setFixedWidth(100);
	
	/* Config Controller Type */
	controllerType = new QComboBox;
	controllerType->addItem("TEST",0); 
	controllerType->addItem("PID",1); 
	controllerType->addItem("OTHER",2); 
	controllerType->setCurrentIndex(1); 
	controllerType->setFixedWidth(100);
	controllerType->setEnabled(false);

	/* Config Controller Var */
	
	//Set Control Signal
	controlVar = new QComboBox;
	controlVar->addItem("Position",0); 
	controlVar->addItem("Speed",1); 
	//controlVar->addItem("Current",2); 
	controlVar->setCurrentIndex(0); 
	controlVar->setFixedWidth(100);
	
	generalLayout->addWidget(new QLabel(tr("Sampling Period. :")), 0, 0);
	generalLayout->addWidget(Tsampling,0,1);
	generalLayout->addWidget(new QLabel(tr("Controller Type:")), 0, 2);
	generalLayout->addWidget(controllerType,0,3);
	generalLayout->addWidget(new QLabel(tr("Control Var. :")), 1, 0);
	generalLayout->addWidget(controlVar,1,1);


	/* Reference */
	//QLabel *referenceTypeLabel = new QLabel(tr("Reference Type"));
	referenceType = new QComboBox;
	referenceType->addItem("NONE",0); 
	referenceType->addItem("DELTA",DELTA); 
	referenceType->addItem("STEP",STEP); 
	referenceType->addItem("RAMP",RAMP); 
	referenceType->addItem("PARAB",PARABOLA); 
	referenceType->addItem("SIN",SIN); 
	referenceType->addItem("COS",COS); 
	referenceType->addItem("TRAP",TRAPEZOIDAL); 
	referenceType->setCurrentIndex(2); 
	referenceType->setFixedWidth(100);
	connect(referenceType, SIGNAL(currentIndexChanged(int)),this,SLOT(handleReferenceTypeChanged(int)));	
	
	referenceValue1 = new QDoubleSpinBox;
	referenceValue1->setRange(-500, 500);
	referenceValue1->setSingleStep(0.01);
	referenceValue1->setValue(3.1415);
	referenceValue1->setLocale(QLocale(QLocale::English));
	referenceValue1->setFixedWidth(100);
	
	referenceValue2 = new QDoubleSpinBox;
	referenceValue2->setRange(-500, 500);
	referenceValue2->setSingleStep(0.01);
	referenceValue2->setValue(0.00);
	referenceValue2->setLocale(QLocale(QLocale::English));
	referenceValue2->setFixedWidth(100);
	referenceValue2->setEnabled(false);
	
	referenceValue3 = new QDoubleSpinBox;
	referenceValue3->setRange(-500, 500);
	referenceValue3->setSingleStep(0.01);
	referenceValue3->setValue(0.00);
	referenceValue3->setLocale(QLocale(QLocale::English));
	referenceValue3->setFixedWidth(100);
	referenceValue3->setEnabled(false);
	
	referenceValue4 = new QDoubleSpinBox;
	referenceValue4->setRange(-500, 500);
	referenceValue4->setSingleStep(0.01);
	referenceValue4->setValue(0.00);
	referenceValue4->setLocale(QLocale(QLocale::English));
	referenceValue4->setFixedWidth(100);
	referenceValue4->setEnabled(false);
	
	generalLayout->addWidget(new QLabel(tr("Ref. Type.:")), 1, 2);
	generalLayout->addWidget(referenceType, 1, 3);
	generalLayout->addWidget(new QLabel(tr("Ref. Value 1.:")), 2, 0);
	generalLayout->addWidget(referenceValue1, 2, 1);
	generalLayout->addWidget(new QLabel(tr("Ref. Value 2.:")), 2, 2);
	generalLayout->addWidget(referenceValue2, 2, 3);
	generalLayout->addWidget(new QLabel(tr("Ref. Value 3.:")), 3, 0);
	generalLayout->addWidget(referenceValue3, 3, 1);
	generalLayout->addWidget(new QLabel(tr("Ref. Value 4.:")), 3, 2);
	generalLayout->addWidget(referenceValue4, 3, 3);

	generalGroup->setLayout(generalLayout);
	
	//Param Data
	//CreateControllerParams
	QHBoxLayout *mainLayout = new QHBoxLayout;
	for ( int i = 0 ; i < NUM_CONTROLLER_LOOPS ; i++ )
	{
		createController(i);
		mainLayout->addWidget(controllerGroup[i]);
	}

	mainLayout->addStretch(0.5);
	
	QGroupBox *paramGroup = new QGroupBox();
	paramGroup->setLayout(mainLayout);

	//Final Layout
	QVBoxLayout *finalLayout = new QVBoxLayout;
	finalLayout->addWidget(generalGroup);
	finalLayout->addWidget(paramGroup);
	finalLayout->addStretch(-0.5);	
	setLayout(finalLayout);

}

/*********************************************
 **********************************************/

void ControllerMotorTab::createController( unsigned int num_controller )
{

	//Create controller Group with name
	controllerGroup[num_controller] = new QGroupBox(controllersName[num_controller]);

	QLabel* controllerParamLabel[NUM_CONTROLLER_PARAM];
	
	//For every param of controller create param spinbox and label
	for ( int num_param = 0 ; num_param < NUM_CONTROLLER_PARAM ; num_param++ )
	{
		controllerParamLabel[num_param] = new QLabel(controllerParamsNamesGlobal[num_param]);
		controllerParam[num_controller][num_param] = new QDoubleSpinBox;
		controllerParam[num_controller][num_param]->setRange(-999.999, 999.999);
		controllerParam[num_controller][num_param]->setSingleStep(0.001);
		controllerParam[num_controller][num_param]->setLocale(QLocale(QLocale::English));
		if (num_controller == DI && num_param == KP)
			controllerParam[num_controller][num_param]->setValue(1.0);
		else if (num_param ==  WUP )
		{
			controllerParam[num_controller][num_param]->setValue(12.0);
			controllerParam[num_controller][num_param]->setEnabled(false);
		}

		else
			controllerParam[num_controller][num_param]->setValue(0.000);
		controllerParam[num_controller][num_param]->setFixedWidth(85);
		controllerParam[num_controller][num_param]->setDecimals(3);
	}

	//Create controller layout and add controller params spinboxes and labels to it
	QVBoxLayout *controllerLayout = new QVBoxLayout;
	for ( int num_param = 0 ; num_param < NUM_CONTROLLER_PARAM ; num_param++)
	{
		controllerLayout->addWidget(controllerParamLabel[num_param]);
		controllerLayout->addWidget(controllerParam[num_controller][num_param]);
	}

	//Add layout to controller group
	controllerGroup[num_controller]->setLayout(controllerLayout);
}

/*********************************************
 **********************************************/

float ControllerTab::getControllerParamValue ( unsigned char u_motor , unsigned char u_controller, unsigned char u_param) 
{
	return controllerMotorTab[u_motor]->controllerParam[u_controller][u_param]->value();
}

/*********************************************
 **********************************************/

unsigned char ControllerTab::getNumControllers ( void )
{
	return NUM_CONTROLLER_LOOPS;
}

/*********************************************
 **********************************************/

unsigned char ControllerTab::getNumParams ( void )
{
	return NUM_CONTROLLER_PARAM;
}

/*********************************************
 **********************************************/

char* ControllerTab::getControllerName ( unsigned char u_controller )
{
	//GetIP
	QString nameString = controllersName[u_controller];
	QByteArray tmp = nameString.toLatin1();
	char *name = tmp.data();
	return name;
}

/*********************************************
 **********************************************/

char* ControllerTab::getControllerParamName ( unsigned char u_param )
{
	//GetIP
	QString nameString = controllerParamsNamesGlobal[u_param];
	QByteArray tmp = nameString.toLatin1();
	char *name = tmp.data();
	return name;
}

/*********************************************
 **********************************************/
unsigned char ControllerTab::getControllerType ( unsigned char u_motor )
{
	return controllerMotorTab[u_motor]->controllerType->currentIndex();
}

/*********************************************
 **********************************************/
unsigned int ControllerTab::getControllerSamplingFreq ( unsigned char u_motor )
{ 
	float fSamplingPeriod = (float) controllerMotorTab[u_motor]->Tsampling->value(); //in ms
	//fSamplingPeriod /= 1000.0; //go for secs
	unsigned int uSamplingFreq = (unsigned int)  (( 1.0 / fSamplingPeriod ) * 1000); //inHz

	return uSamplingFreq;
		
	//return controllerMotorTab[u_motor]->Tsampling->value();
}

/*********************************************
 **********************************************/
unsigned char ControllerTab::getControllerControlVar ( unsigned char u_motor )
{
	return controllerMotorTab[u_motor]->controlVar->currentIndex();
}

/*********************************************
 **********************************************/
unsigned char ControllerTab::getControllerRefType ( unsigned char u_motor )
{
	return controllerMotorTab[u_motor]->referenceType->currentIndex();
}
/*********************************************
 **********************************************/
float ControllerTab::getControllerRefValue1 ( unsigned char u_motor )
{
	return controllerMotorTab[u_motor]->referenceValue1->value();
}

/*********************************************
 **********************************************/

float ControllerTab::getControllerRefValue2 ( unsigned char u_motor )
{
	return controllerMotorTab[u_motor]->referenceValue2->value();
}

/*********************************************
 **********************************************/
float ControllerTab::getControllerRefValue3 ( unsigned char u_motor )
{
	return controllerMotorTab[u_motor]->referenceValue3->value();
}

/*********************************************
 **********************************************/

float ControllerTab::getControllerRefValue4 ( unsigned char u_motor )
{
	return controllerMotorTab[u_motor]->referenceValue4->value();
}

/*********************************************
 **********************************************/

void ControllerTab::enableController ( unsigned char u_motor )
{
	//ControllersTab->setTabEnabled(u_motor,1);
}

/*********************************************
 **********************************************/

void ControllerTab::disableController ( unsigned char u_motor )
{
	//ControllersTab->setTabEnabled(u_motor,0);
}

/*********************************************
 **********************************************/

void ControllerTab::setControllerSamplingFreq	( unsigned char u_motor, unsigned int u_freq)
{
	float fSamplingPeriod = (float) ( 1.0 / (float) u_freq ); // in sec
	fSamplingPeriod *= 1000; //go for msec
	
	controllerMotorTab[u_motor]->Tsampling->setValue((unsigned int) fSamplingPeriod);
}
/*********************************************
 **********************************************/

void ControllerTab::setControllerType					( unsigned char u_motor, unsigned char u_type )
{
	controllerMotorTab[u_motor]->controllerType->setCurrentIndex(u_type);
}
/*********************************************
 **********************************************/

void ControllerTab::setControllerControlVar 	( unsigned char u_motor, unsigned char u_control_var )
{
	controllerMotorTab[u_motor]->controlVar->setCurrentIndex(u_control_var);
}
/*********************************************
 **********************************************/

void ControllerTab::setControllerRefType			( unsigned char u_motor, unsigned char u_ref_type )
{
	controllerMotorTab[u_motor]->referenceType->setCurrentIndex(u_ref_type);
}

/*********************************************
 **********************************************/

void ControllerTab::setControllerRefValue1		( unsigned char u_motor, float f_ref_value )
{
	controllerMotorTab[u_motor]->referenceValue1->setValue(f_ref_value);
}
/*********************************************
 **********************************************/
void ControllerTab::setControllerRefValue2		( unsigned char u_motor, float f_ref_value )
{
	controllerMotorTab[u_motor]->referenceValue2->setValue(f_ref_value);
}
/*********************************************
 **********************************************/
void ControllerTab::setControllerRefValue3		( unsigned char u_motor, float f_ref_value )
{
	controllerMotorTab[u_motor]->referenceValue3->setValue(f_ref_value);
}
/*********************************************
 **********************************************/
void ControllerTab::setControllerRefValue4		( unsigned char u_motor, float f_ref_value )
{
	controllerMotorTab[u_motor]->referenceValue4->setValue(f_ref_value);
}
/*********************************************
 **********************************************/
void  ControllerTab::setControllerParamValue 	( unsigned char u_motor , unsigned char u_controller, 
																	unsigned char u_param, float f_value)
{
	controllerMotorTab[u_motor]->controllerParam[u_controller][u_param]->setValue(f_value);
}
/*********************************************
 **********************************************/
void ControllerMotorTab::handleReferenceTypeChanged(int index)
{
	if (index == DELTA || index == STEP || index == RAMP || index == PARABOLA )
	{
		referenceValue1->setEnabled(true);
		referenceValue2->setEnabled(false);
		referenceValue3->setEnabled(false);
		referenceValue4->setEnabled(false);
	}
	else if (index == SIN || index == COS )
	{
		referenceValue1->setEnabled(true);
		referenceValue2->setEnabled(true);
		referenceValue3->setEnabled(false);
		referenceValue4->setEnabled(false);
	}
	else if (index == TRAPEZOIDAL )
	{
		referenceValue1->setEnabled(true);
		referenceValue2->setEnabled(true);
		referenceValue3->setEnabled(true);
		referenceValue4->setEnabled(true);
	}
	else
	{
		referenceValue1->setEnabled(false);
		referenceValue2->setEnabled(false);
		referenceValue3->setEnabled(false);
		referenceValue4->setEnabled(false);
	}
}
