#include <QtGui>

#include "mainwindow.h"

MainWindow::MainWindow(PlotWindow *q_plot, NetworkTab *network_tab, ControllerTab *controller_tab, PerturbationTab *perturbation_tab, MotorTab *motor_tab, CommunicationTab *communication_tab)
{
	createActions();
	createMenus();

	networkTab = network_tab;
	controllerTab = controller_tab;
	perturbationTab = perturbation_tab;
	motorTab = motor_tab;
	communicationTab = communication_tab;
	plot = q_plot;
	//setMinimumSize(160, 160);
	//resize(480, 320);
}

void MainWindow::contextMenuEvent(QContextMenuEvent *event)
{
	QMenu menu(this);
	menu.addAction(cutAct);
	menu.addAction(copyAct);
	menu.addAction(pasteAct);
	menu.exec(event->globalPos());

}

void MainWindow::newFile()
{
	//infoLabel->setText(tr("Invoked <b>File|New</b>"));
}

void MainWindow::open()
{
	QString QTFileName = QFileDialog::getOpenFileName(this, tr("Load File"),".",tr("XML files (*.xml)"));
	QByteArray tmp = QTFileName.toLatin1();
	const char *filename = tmp.data();

	printf("Reading XML file: %s\n", filename);
	/* Create and XML Doc */
	XMLDocument conf;	
	/* Read xml file*/
	if (!conf.LoadFile(filename))
	{
		/* Get root of file */
		XMLElement* root = conf.FirstChildElement();
		string elemName;
		const char* attr;
		/* Go for all elements */
		for( XMLElement* elem = root->FirstChildElement() ; elem != NULL ; elem = elem->NextSiblingElement() )
		{
			/*** NETWORK ***/
			if ( (elemName = elem->Value())== "Network" )
			{
				networkTab->setUser (elem->Attribute("user"));
				networkTab->setWorkStation ((unsigned int) (atoi(elem->Attribute("workstation"))));
				networkTab->setFilePath 	(elem->Attribute("path"));
				networkTab->setFilePrefix (elem->Attribute("prefix"));
			}//end Network

			/*** MOTORS ***/
			else if (elemName == "Motors" )
			{
				for( XMLElement* e = elem->FirstChildElement() ; e != NULL ; e = e->NextSiblingElement() )
				{
					unsigned int num_motor = atoi(e->Attribute("num"));

					if ( atoi ( e->Attribute ( "active" ) ) )
					{
						motorTab->enableMotor(num_motor);
						motorTab->setMotorPWMFreq(num_motor, atoi ( e->Attribute ("freq")));
					}
					else
					{
						motorTab->disableMotor(num_motor);
					}
				}
			}//end Motors

			/*** CONTROLLERS **/
			else if (elemName == "Controllers" )
			{
				for( XMLElement* e = elem->FirstChildElement() ; e != NULL ; e = e->NextSiblingElement() )
				{
					unsigned int num_controller = atoi(e->Attribute("num"));
					if ( atoi ( e->Attribute ( "active" ) ) )
					{
						controllerTab->enableController(num_controller);
						controllerTab->setControllerSamplingFreq ( num_controller , atoi(e->Attribute("freq")));
						controllerTab->setControllerType ( num_controller, atoi(e->Attribute("type")));
						controllerTab->setControllerControlVar ( num_controller, atoi(e->Attribute("controlVar")));
						controllerTab->setControllerRefType ( num_controller, atoi(e->Attribute("refType")));
						if ( (attr = e->Attribute("refValue1")) )
							controllerTab->setControllerRefValue1 ( num_controller, atof(attr));
						if ( (attr = e->Attribute("refValue2")) )
							controllerTab->setControllerRefValue2	( num_controller, atof(attr));
						if ( (attr = e->Attribute("refValue3")) )
							controllerTab->setControllerRefValue3	( num_controller, atof(attr));
						if ( (attr = e->Attribute("refValue4")) )
							controllerTab->setControllerRefValue4	( num_controller, atof(attr));
						
						/* Go for controllers loops */
						for( XMLElement* l = e->FirstChildElement() ; l != NULL ; l = l->NextSiblingElement() )
						{
							unsigned int num_loop = atoi(l->Attribute("num"));
							if ( (attr = l->Attribute("Kp")) )
								controllerTab->setControllerParamValue ( num_controller, num_loop, KP, atof(attr));
							if ( (attr = l->Attribute("Ki")) )
								controllerTab->setControllerParamValue ( num_controller, num_loop, KI, atof(attr));
							if ( (attr = l->Attribute("Kd")) )
								controllerTab->setControllerParamValue ( num_controller, num_loop, KD, atof(attr));
							if ( (attr = l->Attribute("Wup")) )
								controllerTab->setControllerParamValue ( num_controller, num_loop, WUP, atof(attr));
							if ( (attr = l->Attribute("Ep")) )
								controllerTab->setControllerParamValue ( num_controller, num_loop, EP, atof(attr));
						}//end of controller loops

					}
					else
					{
						controllerTab->disableController(num_controller);
					}

				}//end motors in controllers

			}//end Controllers
			
			/*** PERTURBATIONS **/
			else if (elemName == "Perturbations" )
			{
				for( XMLElement* e = elem->FirstChildElement() ; e != NULL ; e = e->NextSiblingElement() )
				{
					unsigned int num_motor = atoi(e->Attribute("num"));
					if ( atoi ( e->Attribute ( "active" ) ) )
					{
						printf("MOTOR: %d\n", num_motor);
						perturbationTab->setPerturbationRefType ( num_motor, atoi(e->Attribute("perType")));
						if ( (attr = e->Attribute("perValue1")) )
							perturbationTab->setPerturbationRefValue1 ( num_motor, atof(attr));
						if ( (attr = e->Attribute("perValue2")) )
							perturbationTab->setPerturbationRefValue2	( num_motor, atof(attr));
						if ( (attr = e->Attribute("perValue3")) )
							perturbationTab->setPerturbationRefValue3	( num_motor, atof(attr));
						if ( (attr = e->Attribute("perValue4")) )
							perturbationTab->setPerturbationRefValue4	( num_motor, atof(attr));
						
					}
				}//end motors in perturbations
			}//end perturbations

			/*** COMMUNICATION ***/
			else if (elemName == "Communication" )
			{
				for( XMLElement* e = elem->FirstChildElement() ; e != NULL ; e = e->NextSiblingElement() )
				{
					unsigned int num_motor = atoi(e->Attribute("num"));
					if ( atoi ( e->Attribute ( "active" ) ) )
					{
						communicationTab->enableCommunication(num_motor);
						communicationTab->setCommunicationTransmissionFreq(num_motor, atoi ( e->Attribute ("freq")));
						communicationTab->setCommunicationTimeLimit(num_motor, atof ( e->Attribute ("timeLimit")));
						communicationTab->setCommunicationParamValue(num_motor, POS, atoi (e->Attribute("pos")));
						communicationTab->setCommunicationParamValue(num_motor, SPEED, atoi (e->Attribute("vel")));
						communicationTab->setCommunicationParamValue(num_motor, CURRENT, atoi (e->Attribute("cur")));
						communicationTab->setCommunicationParamValue(num_motor, REF, atoi (e->Attribute("ref")));
						communicationTab->setCommunicationParamValue(num_motor, ERROR, atoi (e->Attribute("err")));
						communicationTab->setCommunicationParamValue(num_motor, ERROR_REF_FB, atoi (e->Attribute("err_ref_fb")));
						communicationTab->setCommunicationParamValue(num_motor, UFF, atoi (e->Attribute("uFF")));
						communicationTab->setCommunicationParamValue(num_motor, UDI, atoi (e->Attribute("uDI")));
						communicationTab->setCommunicationParamValue(num_motor, UFB, atoi (e->Attribute("uFB")));
						communicationTab->setCommunicationParamValue(num_motor, UPA, atoi (e->Attribute("uPA")));
						communicationTab->setCommunicationParamValue(num_motor, U, atoi (e->Attribute("u")));
						communicationTab->setCommunicationParamValue(num_motor, USAT, atoi (e->Attribute("uSat")));
						communicationTab->setCommunicationParamValue(num_motor, Y, atoi (e->Attribute("y")));
					}
					else
					{
						communicationTab->disableCommunication(num_motor);
					}
				}
			}
		}//end XML
	}
	printf("End Reading XML file\n");
}

void MainWindow::save()
{
	XMLDocument doc;
	XMLElement* mainTag = doc.NewElement("secoApp"); 
	XMLElement* tagName; 

	/*** HEADER ***/
	//tagName = doc.NewElement("?");
	//tagName->SetAttribute("xml version", "1.0");
	//doc.InsertEndChild(tagName);

	/*** GENERAL ***/
	tagName = doc.NewElement("Network");
	
	//Get user
	QString userQString = networkTab->getUser();
	QByteArray tmpUser = userQString.toLatin1();
	const char *user = tmpUser.data();

	tagName->SetAttribute("user", user);
	
	//Get Workstation
	unsigned int workstation = networkTab->getWorkStation();
	
	tagName->SetAttribute("workstation", workstation);
	

	//Get FilePath
	QString pathQString = networkTab->getFilePath();
	QByteArray tmpPath = pathQString.toLatin1();
	const char *path = tmpPath.data();

	tagName->SetAttribute("path", path);
	//Get FilePath
	QString prefixQString = networkTab->getFilePrefix();
	QByteArray tmpPrefix = prefixQString.toLatin1();
	const char *prefix = tmpPrefix.data();

	tagName->SetAttribute("prefix", prefix);
	
	mainTag->InsertEndChild(tagName);


	/*** MOTORS ***/
	tagName = doc.NewElement("Motors");

	unsigned char num_motors = motorTab->getNumMotors();
	for (int u_motor = 0 ; u_motor < num_motors ; u_motor++ )
	{
		XMLElement* subtagName = doc.NewElement("motor"); 
		subtagName->SetAttribute("num", u_motor);
		subtagName->SetAttribute("active", motorTab->getMotorStatus(u_motor));
		subtagName->SetAttribute("freq", motorTab->getMotorPWMFreq(u_motor));

		tagName->InsertEndChild(subtagName);
	}

	mainTag->InsertEndChild(tagName);

	/*** CONTROLLERS ***/
	tagName = doc.NewElement("Controllers");
	unsigned char num_controllers = controllerTab->getNumControllers();
	unsigned char num_params = controllerTab->getNumParams();

	//Check for motors
	for ( int u_motor = 0 ;  u_motor < num_motors ; u_motor++ ) 
	{
		XMLElement* subtagName = doc.NewElement("controller"); 
		subtagName->SetAttribute("num", u_motor);
		subtagName->SetAttribute("active", motorTab->getMotorStatus(u_motor));
		subtagName->SetAttribute("freq", controllerTab->getControllerSamplingFreq(u_motor));
		subtagName->SetAttribute("type", controllerTab->getControllerType(u_motor));
		subtagName->SetAttribute("controlVar", controllerTab->getControllerControlVar(u_motor));
		subtagName->SetAttribute("refType", controllerTab->getControllerRefType(u_motor));
		subtagName->SetAttribute("refValue1", controllerTab->getControllerRefValue1(u_motor));
		subtagName->SetAttribute("refValue2", controllerTab->getControllerRefValue2(u_motor));
		subtagName->SetAttribute("refValue3", controllerTab->getControllerRefValue3(u_motor));
		subtagName->SetAttribute("refValue4", controllerTab->getControllerRefValue4(u_motor));
		
		for ( int u_loop = 0 ; u_loop < num_controllers; u_loop++ )
		{
			XMLElement* subsubtagName = doc.NewElement("loop"); 
			subsubtagName->SetAttribute("num", u_loop);
			for (int u_param = 0 ; u_param < num_params ; u_param++ )
			{
				subsubtagName->SetAttribute(controllerTab->getControllerParamName(u_param), controllerTab->getControllerParamValue(u_motor, u_loop, u_param));
			}//end params

			subtagName->InsertEndChild(subsubtagName);
		}//end loops
		tagName->InsertEndChild(subtagName);
	}//end controller of motor
	
	mainTag->InsertEndChild(tagName);

	/*** PERTURBATIONS ***/
	tagName = doc.NewElement("Perturbations");
	//unsigned char num_perturbations = controllerTab->getNumControllers();
	//unsigned char num_params = controllerTab->getNumParams();

	//Check for motors
	for ( int u_motor = 0 ;  u_motor < num_motors ; u_motor++ ) 
	{
		XMLElement* subtagName = doc.NewElement("perturbation"); 
		subtagName->SetAttribute("num", u_motor);
		subtagName->SetAttribute("active", motorTab->getMotorStatus(u_motor));
		subtagName->SetAttribute("perType", perturbationTab->getPerturbationRefType(u_motor));
		subtagName->SetAttribute("perValue1", perturbationTab->getPerturbationRefValue1(u_motor));
		subtagName->SetAttribute("perValue2", perturbationTab->getPerturbationRefValue2(u_motor));
		subtagName->SetAttribute("perValue3", perturbationTab->getPerturbationRefValue3(u_motor));
		subtagName->SetAttribute("perValue4", perturbationTab->getPerturbationRefValue4(u_motor));
		
		tagName->InsertEndChild(subtagName);
	}//end controller of motor
	
	mainTag->InsertEndChild(tagName);
	
	/*** COMMUNICATION ***/
	tagName = doc.NewElement("Communication");
	num_params = communicationTab->getNumParams();
	//Check for motors
	for ( int u_motor = 0 ;  u_motor < num_motors ; u_motor++ ) 
	{
		XMLElement* subtagName = doc.NewElement("comm"); 
		subtagName->SetAttribute("num", u_motor);
		subtagName->SetAttribute("active", motorTab->getMotorStatus(u_motor));
		subtagName->SetAttribute("freq", communicationTab->getCommunicationTransmissionFreq(u_motor));
		subtagName->SetAttribute("timeLimit", communicationTab->getCommunicationTimeLimit(u_motor));
		for (int u_param = 0 ; u_param < num_params ; u_param++ )
		{
			subtagName->SetAttribute(communicationTab->getCommunicationParamName(u_param), communicationTab->getCommunicationParamValue(u_motor, u_param));
		}//end param

		tagName->InsertEndChild(subtagName);
	}//end communication

	mainTag->InsertEndChild(tagName);

	doc.InsertEndChild(mainTag);
	
	//XMLSave
	QString QTFileName = QFileDialog::getSaveFileName(this, tr("Save File"),"controller.xml",tr("XML files (*.xml)"));
	QByteArray tmp = QTFileName.toLatin1();
	char *filename = tmp.data();
	doc.SaveFile(filename, false);
}

void MainWindow::savePdf()
{
	QString QTFileName = QFileDialog::getSaveFileName(this, tr("Save plot"),"plot.pdf",tr("PDF files (*.pdf)"));
	plot->savePdf(QTFileName);
}

void MainWindow::loadPlots()
{
	/* Get a list of files */
	QStringList files = QFileDialog::getOpenFileNames(this, "Select one or more files to plot",".","");

	/* Clear plot data */
	plot->clearData();
	plot->setTimeLimit(1000);

	unsigned char uVar		= 0;
	unsigned char uMotor	= MOTOR3;

	/* Iterate over the files */
	for (int i = 0; i < files.size(); ++i)
	{
		/* Get variable to plot */
		if ( strstr(files.at(i).toLocal8Bit().constData(), "POS") != NULL)
			uVar = POS;
		else if ( strstr(files.at(i).toLocal8Bit().constData(), "VEL") != NULL)
			uVar = SPEED;
		else if ( strstr(files.at(i).toLocal8Bit().constData(), "CUR") != NULL)
			uVar = CURRENT;
		else if ( strstr(files.at(i).toLocal8Bit().constData(), "REF") != NULL)
			uVar = REF;
		else if ( strstr(files.at(i).toLocal8Bit().constData(), "ERR") != NULL)
			uVar = ERROR;
		else if ( strstr(files.at(i).toLocal8Bit().constData(), "U") != NULL)
			uVar = U;
	
		/* Set Var is active */
		plot->setCommVarFlag(uVar, true);

		printf("Loading: %s\n", files.at(i).toLocal8Bit().constData());
		/* Open file */
		std::ifstream file(files.at(i).toLocal8Bit().constData());
		/* Read Line by Line */
		std::string sTime; 
		std::string sValue; 
		while (std::getline(file, sTime, ' '))
		{
			getline(file,sValue);
			size_t f = sTime.find(".");
			sTime.replace(f, 1, ",");
			f = sValue.find(".");
			sValue.replace(f, 1, ",");

			plot->addValue(uMotor, uVar, atof(sTime.c_str()), atof(sValue.c_str()));
		}
	}
	
	plot->setupPlot();
}

/***************************************
 * ************************************/

void MainWindow::print()
{
	//infoLabel->setText(tr("Invoked <b>File|Print</b>"));
}

void MainWindow::undo()
{
	//infoLabel->setText(tr("Invoked <b>Edit|Undo</b>"));
}

void MainWindow::redo()
{
	//infoLabel->setText(tr("Invoked <b>Edit|Redo</b>"));
}

void MainWindow::cut()
{
	//infoLabel->setText(tr("Invoked <b>Edit|Cut</b>"));
}

void MainWindow::copy()
{
	//infoLabel->setText(tr("Invoked <b>Edit|Copy</b>"));
}

void MainWindow::paste()
{
	//infoLabel->setText(tr("Invoked <b>Edit|Paste</b>"));
}

void MainWindow::bold()
{
	//infoLabel->setText(tr("Invoked <b>Edit|Format|Bold</b>"));
}

void MainWindow::italic()
{
	//infoLabel->setText(tr("Invoked <b>Edit|Format|Italic</b>"));
}

void MainWindow::leftAlign()
{
	//infoLabel->setText(tr("Invoked <b>Edit|Format|Left Align</b>"));
}

void MainWindow::rightAlign()
{
	//infoLabel->setText(tr("Invoked <b>Edit|Format|Right Align</b>"));
}

void MainWindow::justify()
{
	//infoLabel->setText(tr("Invoked <b>Edit|Format|Justify</b>"));
}

void MainWindow::center()
{
	//infoLabel->setText(tr("Invoked <b>Edit|Format|Center</b>"));
}

void MainWindow::setLineSpacing()
{
	//infoLabel->setText(tr("Invoked <b>Edit|Format|Set Line Spacing</b>"));
}

void MainWindow::setParagraphSpacing()
{
	//infoLabel->setText(tr("Invoked <b>Edit|Format|Set Paragraph Spacing</b>"));
}

void MainWindow::about()
{
	//infoLabel->setText(tr("Invoked <b>Help|About</b>"));
	QMessageBox::about(this, tr("About Menu"),
			tr("The <b>Menu</b> example shows how to create "
				"menu-bar menus and context menus."));
}

void MainWindow::aboutQt()
{
	infoLabel->setText(tr("Invoked <b>Help|About Qt</b>"));
}

void MainWindow::createActions()
{
	//newAct = new QAction(tr("&New"), this);
	//newAct->setShortcuts(QKeySequence::New);
	//newAct->setStatusTip(tr("Create a new file"));
	//connect(newAct, SIGNAL(triggered()), this, SLOT(newFile()));

	openAct = new QAction(tr("&Load"), this);
	openAct->setShortcuts(QKeySequence::Open);
	openAct->setStatusTip(tr("Open an existing file"));
	connect(openAct, SIGNAL(triggered()), this, SLOT(open()));

	saveAct = new QAction(tr("&Save"), this);
	saveAct->setShortcuts(QKeySequence::Save);
	saveAct->setStatusTip(tr("Save the document to disk"));
	connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));
	
	savePdfAct = new QAction(tr("&Save"), this);
	savePdfAct->setShortcuts(QKeySequence::Print);
	savePdfAct->setStatusTip(tr("Save plot to disk"));
	connect(savePdfAct, SIGNAL(triggered()), this, SLOT(savePdf()));
	
	loadPlotAct = new QAction(tr("&Load"), this);
	loadPlotAct->setShortcuts(QKeySequence::Bold);
	loadPlotAct->setStatusTip(tr("Load plot files"));
	connect(loadPlotAct, SIGNAL(triggered()), this, SLOT(loadPlots()));

	//printAct = new QAction(tr("&Print..."), this);
	//printAct->setShortcuts(QKeySequence::Print);
	//printAct->setStatusTip(tr("Print the document"));
	//connect(printAct, SIGNAL(triggered()), this, SLOT(print()));

	//exitAct = new QAction(tr("E&xit"), this);
	//exitAct->setShortcuts(QKeySequence::Quit);
	//exitAct->setStatusTip(tr("Exit the application"));
	//connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

	//undoAct = new QAction(tr("&Undo"), this);
	//undoAct->setShortcuts(QKeySequence::Undo);
	//undoAct->setStatusTip(tr("Undo the last operation"));
	//connect(undoAct, SIGNAL(triggered()), this, SLOT(undo()));

	//redoAct = new QAction(tr("&Redo"), this);
	//redoAct->setShortcuts(QKeySequence::Redo);
	//redoAct->setStatusTip(tr("Redo the last operation"));
	//connect(redoAct, SIGNAL(triggered()), this, SLOT(redo()));

	//cutAct = new QAction(tr("Cu&t"), this);
	//cutAct->setShortcuts(QKeySequence::Cut);
	//cutAct->setStatusTip(tr("Cut the current selection's contents to the "
	//"clipboard"));
	//connect(cutAct, SIGNAL(triggered()), this, SLOT(cut()));

	//copyAct = new QAction(tr("&Copy"), this);
	//copyAct->setShortcuts(QKeySequence::Copy);
	//copyAct->setStatusTip(tr("Copy the current selection's contents to the "
	//"clipboard"));
	//connect(copyAct, SIGNAL(triggered()), this, SLOT(copy()));

	//pasteAct = new QAction(tr("&Paste"), this);
	//pasteAct->setShortcuts(QKeySequence::Paste);
	//pasteAct->setStatusTip(tr("Paste the clipboard's contents into the current "
	//"selection"));
	//connect(pasteAct, SIGNAL(triggered()), this, SLOT(paste()));

	//boldAct = new QAction(tr("&Bold"), this);
	//boldAct->setCheckable(true);
	//boldAct->setShortcut(QKeySequence::Bold);
	//boldAct->setStatusTip(tr("Make the text bold"));
	//connect(boldAct, SIGNAL(triggered()), this, SLOT(bold()));

	//QFont boldFont = boldAct->font();
	//boldFont.setBold(true);
	//boldAct->setFont(boldFont);

	//italicAct = new QAction(tr("&Italic"), this);
	//italicAct->setCheckable(true);
	//italicAct->setShortcut(QKeySequence::Italic);
	//italicAct->setStatusTip(tr("Make the text italic"));
	//connect(italicAct, SIGNAL(triggered()), this, SLOT(italic()));

	//QFont italicFont = italicAct->font();
	//italicFont.setItalic(true);
	//italicAct->setFont(italicFont);

	//setLineSpacingAct = new QAction(tr("Set &Line Spacing..."), this);
	//setLineSpacingAct->setStatusTip(tr("Change the gap between the lines of a "
	//"paragraph"));
	//connect(setLineSpacingAct, SIGNAL(triggered()), this, SLOT(setLineSpacing()));

	//setParagraphSpacingAct = new QAction(tr("Set &Paragraph Spacing..."), this);
	//setLineSpacingAct->setStatusTip(tr("Change the gap between paragraphs"));
	//connect(setParagraphSpacingAct, SIGNAL(triggered()),
	//this, SLOT(setParagraphSpacing()));

	//aboutAct = new QAction(tr("&About"), this);
	//aboutAct->setStatusTip(tr("Show the application's About box"));
	//connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

	//aboutQtAct = new QAction(tr("About &Qt"), this);
	//aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
	//connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
	//connect(aboutQtAct, SIGNAL(triggered()), this, SLOT(aboutQt()));

	//leftAlignAct = new QAction(tr("&Left Align"), this);
	//leftAlignAct->setCheckable(true);
	//leftAlignAct->setShortcut(tr("Ctrl+L"));
	//leftAlignAct->setStatusTip(tr("Left align the selected text"));
	//connect(leftAlignAct, SIGNAL(triggered()), this, SLOT(leftAlign()));

	//rightAlignAct = new QAction(tr("&Right Align"), this);
	//rightAlignAct->setCheckable(true);
	//rightAlignAct->setShortcut(tr("Ctrl+R"));
	//rightAlignAct->setStatusTip(tr("Right align the selected text"));
	//connect(rightAlignAct, SIGNAL(triggered()), this, SLOT(rightAlign()));

	//justifyAct = new QAction(tr("&Justify"), this);
	//justifyAct->setCheckable(true);
	//justifyAct->setShortcut(tr("Ctrl+J"));
	//justifyAct->setStatusTip(tr("Justify the selected text"));
	//connect(justifyAct, SIGNAL(triggered()), this, SLOT(justify()));

	//centerAct = new QAction(tr("&Center"), this);
	//centerAct->setCheckable(true);
	//centerAct->setShortcut(tr("Ctrl+E"));
	//centerAct->setStatusTip(tr("Center the selected text"));
	//connect(centerAct, SIGNAL(triggered()), this, SLOT(center()));

	//alignmentGroup = new QActionGroup(this);
	//alignmentGroup->addAction(leftAlignAct);
	//alignmentGroup->addAction(rightAlignAct);
	//alignmentGroup->addAction(justifyAct);
	//alignmentGroup->addAction(centerAct);
	//leftAlignAct->setChecked(true);
}

void MainWindow::createMenus()
{
	fileMenu = menuBar()->addMenu(tr("&File"));
	fileMenu->addAction(openAct);
	fileMenu->addAction(saveAct);

	fileMenu = menuBar()->addMenu(tr("&Plot"));
	fileMenu->addAction(savePdfAct);
	fileMenu->addAction(loadPlotAct);

	//fileMenu->addAction(newAct);
	//fileMenu->addAction(printAct);
	//fileMenu->addSeparator();
	//fileMenu->addAction(exitAct);

	//editMenu = menuBar()->addMenu(tr("&Edit"));
	//editMenu->addAction(undoAct);
	//editMenu->addAction(redoAct);
	//editMenu->addSeparator();
	//editMenu->addAction(cutAct);
	//editMenu->addAction(copyAct);
	//editMenu->addAction(pasteAct);
	//editMenu->addSeparator();

	//helpMenu = menuBar()->addMenu(tr("&Help"));
	//helpMenu->addAction(aboutAct);
	//helpMenu->addAction(aboutQtAct);

	//formatMenu = editMenu->addMenu(tr("&Format"));
	//formatMenu->addAction(boldAct);
	//formatMenu->addAction(italicAct);
	//formatMenu->addSeparator()->setText(tr("Alignment"));
	//formatMenu->addAction(leftAlignAct);
	//formatMenu->addAction(rightAlignAct);
	//formatMenu->addAction(justifyAct);
	//formatMenu->addAction(centerAct);
	//formatMenu->addSeparator();
	//formatMenu->addAction(setLineSpacingAct);
	//formatMenu->addAction(setParagraphSpacingAct);
}
/******************************************************************************/

