#include "tabdialog.h"

TabDialog::TabDialog(PlotWindow *q_plot, CameraWindow *q_camera, QWidget *parent)
: QDialog(parent)
{
	//QApplication a();
	//a.connect(&a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()));

	
	plot 		= q_plot;
	camera 	= q_camera;

	networkTab 						= new NetworkTab;
	motorTab							= new MotorTab;
	controllerTab 				= new ControllerTab(motorTab);
	perturbationTab 			= new PerturbationTab(motorTab);
	communicationTab 			= new CommunicationTab(motorTab);
	communicationDataTab 	= new CommunicationDataTab;

	tabWidget = new QTabWidget;
	tabWidget->addTab( networkTab, 						tr("General") );
	tabWidget->addTab( motorTab, 							tr("Motor") );
	tabWidget->addTab( controllerTab, 				tr("Controller") );
	tabWidget->addTab( perturbationTab, 			tr("Perturbation") );
	tabWidget->addTab( communicationTab, 			tr("Communication") );
	//tabWidget->addTab( communicationDataTab, 	tr("Data") );

	window = new MainWindow (plot, networkTab, controllerTab, perturbationTab, motorTab, communicationTab);
	
	//We connect the signals closed to correctQuit to send a quit signal to the board
	connect( qApp, SIGNAL(lastWindowClosed()), this, SLOT(correctQuit()) );
	
	createBottomBar();
	
  qtsecosocket = new QtSecoSocket(plot, queueValue, led1, led2, led3, led4, connectButton, runButton, resetButton, disconnectButton, camera);

	QVBoxLayout *mainLayout = new QVBoxLayout;
	mainLayout->addWidget(window);
	mainLayout->addWidget(tabWidget);
	mainLayout->addWidget(bottomBarGroup);
	mainLayout->addStretch(2);
	setLayout(mainLayout);

	setWindowTitle(tr("SECO Application"));
	
	/* Set timer */
	m_TCheckSocketTimer = new QTimer(this);
	connect(m_TCheckSocketTimer, SIGNAL(timeout()), this, SLOT(checkSocketStatus()));
	m_TCheckSocketTimer->start(1000);
	
	m_TCheckDataEndTimer = new QTimer(this);
	connect(m_TCheckDataEndTimer, SIGNAL(timeout()), this, SLOT(checkDataEnd()));
	m_TCheckDataEndTimer->start(100);
}

/****************************************
 ****************************************/
void TabDialog::createBottomBar ()
{
	bottomBarGroup = new QGroupBox(tr(""));
  QGridLayout *Bottomlayout = new QGridLayout;
	
	//QPushButton *quitButton = new QPushButton("Quit", this);
	//quitButton->setGeometry(50, 40, 75, 30);
	
	//printButton = new QPushButton("Print", this);
	//printButton->setGeometry(50, 40, 75, 30);
	
	connectButton = new QPushButton("Send", this);
	connectButton->setGeometry(50, 40, 75, 30);
	
  runButton = new QPushButton("Run", this);
  runButton->setGeometry(0, 0, 0, 0);
  //runButton->setEnabled(false);
	
	resetButton = new QPushButton("Reset", this);
	resetButton->setGeometry(50, 40, 75, 30);
	resetButton->setEnabled(false);
	
	disconnectButton = new QPushButton("Disconnect", this);
	disconnectButton->setGeometry(50, 40, 75, 30);
	disconnectButton->setEnabled(false);
	
	connect(connectButton, SIGNAL(clicked()), this, SLOT(SocketConnect()));
	connect(disconnectButton, SIGNAL(clicked()), this , SLOT(SocketDisconnect()));
  //connect(runButton, SIGNAL(clicked()), this, SLOT(configPlot()));
  connect(runButton, SIGNAL(clicked()), this , SLOT(SocketRun()));
	connect(resetButton, SIGNAL(clicked()), this , SLOT(SocketReset()));

  queueValue   = new QLineEdit("");
  queueValue->setFixedWidth(50);

  led1 = new QLedIndicator(bottomBarGroup);
  led1->setObjectName(QString::fromUtf8("led1"));
  led1->setEnabled(false);
  led1->setChecked(true);
  led1->setOnColor1(QColor(255, 0, 0));
  led1->setOnColor2(QColor(176, 0, 2));
  led1->setOffColor1(QColor(100, 0, 0));
  led1->setOffColor2(QColor(100, 0, 0));
  
  led2 = new QLedIndicator(bottomBarGroup);
  led2->setObjectName(QString::fromUtf8("led2"));
  led2->setEnabled(false);
  led2->setChecked(false);
  led2->setOnColor1(QColor(255, 255, 0));
  led2->setOnColor2(QColor(176, 176, 2));
  led2->setOffColor1(QColor(100, 100, 0));
  led2->setOffColor2(QColor(100, 100, 2));

  led3 = new QLedIndicator(bottomBarGroup);
  led3->setObjectName(QString::fromUtf8("led3"));
  led3->setEnabled(false);
  led3->setChecked(false);
  led3->setOnColor1(QColor(0, 255, 0));
  led3->setOnColor2(QColor(0, 176, 2));
  led3->setOffColor1(QColor(0, 100, 0));
  led3->setOffColor2(QColor(0, 100, 2));


  led4 = new QLedIndicator(bottomBarGroup);
  led4->setObjectName(QString::fromUtf8("led4"));
  led4->setEnabled(false);
  led4->setChecked(false);
  led4->setOnColor1(QColor(0, 100, 255));
  led4->setOnColor2(QColor(0, 100, 176));
  led4->setOffColor1(QColor(0, 0, 100));
  led4->setOffColor2(QColor(0, 0, 100));
  
  Bottomlayout->addWidget(connectButton, 0, 0, 1, 2);
  Bottomlayout->addWidget(resetButton, 0, 2, 1, 2);
  Bottomlayout->addWidget(disconnectButton, 0, 4, 1, 2);
  Bottomlayout->addWidget(new QLabel(tr("Waiting Queue:")), 2, 0);
  Bottomlayout->addWidget(new QLabel(tr(">1")), 1, 2, Qt::AlignHCenter);
  Bottomlayout->addWidget(new QLabel(tr("Next")), 1, 3, Qt::AlignHCenter);
  Bottomlayout->addWidget(new QLabel(tr("Working")), 1, 4, Qt::AlignHCenter);
  Bottomlayout->addWidget(new QLabel(tr("Data Tx")), 1, 5, Qt::AlignHCenter);
  Bottomlayout->addWidget(queueValue, 2, 1);
  Bottomlayout->addWidget(led1, 2, 2);
  Bottomlayout->addWidget(led2, 2, 3);
  Bottomlayout->addWidget(led3, 2, 4);
  Bottomlayout->addWidget(led4, 2, 5);

  bottomBarGroup->setLayout(Bottomlayout);
}

void TabDialog::printData ( void )
{
	printf("------------------------------------------------------------------------------------\n");
	//GetNetwork
	printf("NETWORK:\n");
	//GetIP
	QString ipQString = networkTab->getIP();
	QByteArray tmpIP = ipQString.toLatin1();
	const char *ip = tmpIP.data();
	printf("IP: \t%s\n", ip);
	
	//Get Port
	unsigned int port = networkTab->getPort();
	printf("Port: \t%d\n", port);

	//GetMotor
	printf("\n");
	printf("MOTORS:\n");

	unsigned char num_motors = motorTab->getNumMotors();
	for (int i = 0 ; i < num_motors ; i++ )
	{
		unsigned char motorOn = motorTab->getMotorStatus(i);
		unsigned int motorFreq = motorTab->getMotorPWMFreq(i);
		printf ("Motor %d: \tStatus: %d \tPWM Freq: %d\n", i+1, motorOn, motorFreq);
	}

	//GetControllers
	printf("\n");
	printf("CONTROLLERS:\n");
	
	unsigned char num_controllers = controllerTab->getNumControllers();
	unsigned char num_params = controllerTab->getNumParams();

	//Check for motors
	for ( int u_motor = 0 ;  u_motor < num_motors ; u_motor++ ) 
	{
		//If motor ON
		if ( motorTab->getMotorStatus(u_motor) == true )
		{
			unsigned char controllerType = controllerTab->getControllerType(u_motor);
			unsigned char controllerSamplingFreq = controllerTab->getControllerSamplingFreq(u_motor);
			unsigned char controllerControlVar		= controllerTab->getControllerControlVar(u_motor);
			unsigned char controllerRefType				= controllerTab->getControllerRefType(u_motor);
			float 				refValue1								= controllerTab->getControllerRefValue1(u_motor);
			float 				refValue2								= controllerTab->getControllerRefValue2(u_motor);

			//Go for its params
			printf("MOTOR %d\n", u_motor+1);
			printf("Controller Type: %d \tSamplig Freq: %d\n", controllerType, controllerSamplingFreq  );
			printf("Control Var: %d\t Reference Type: %d\n", controllerControlVar, controllerRefType);
			printf("Reference Value 1: %2f\t Reference Value 2: %2f\t", refValue1, refValue2);

			for ( int u_controller = 0 ; u_controller< num_controllers ; u_controller++ )
			{
				printf("CONTROLLER %s:\t", controllerTab->getControllerName(u_controller));
				for (int u_param = 0 ; u_param < num_params ; u_param++ )
				{
					float var = controllerTab->getControllerParamValue(u_motor, u_controller, u_param);
					printf("%s: %2.2f\t", controllerTab->getControllerParamName(u_param), var);
				}//end param
				printf("\n");
			}//end controller
		}//end if motor
	}//end motor

	//GetCommunication
	printf("\n");
	printf("COMMUNICATION:\n");
	
	num_params = communicationTab->getNumParams();

	//Check for motors
	for ( int u_motor = 0 ;  u_motor < num_motors ; u_motor++ ) 
	{
		//If motor ON
		if ( motorTab->getMotorStatus(u_motor) == true )
		{
			unsigned char communicationTransmissionFreq = communicationTab->getCommunicationTransmissionFreq(u_motor);

			//Go for its params
			printf("MOTOR %d \tTransmission Freq: %d\n", u_motor+1, communicationTransmissionFreq  );

			for (int u_param = 0 ; u_param < num_params ; u_param++ )
			{
				printf("%s: %d\t", communicationTab->getCommunicationParamName(u_param), communicationTab->getCommunicationParamValue(u_motor, u_param));
			}//end param
			printf("\n");
		}//end if motor
	}//end motor

	printf("------------------------------------------------------------------------------------\n");

}

/*********************************
 *********************************/

void TabDialog::SocketConnect ( void )
{

	if ( qtsecosocket->connectToServer(networkTab->getIP(), networkTab->getPort(), communicationDataTab )) 
	{
		printf("Connected\n");
		connectButton->setEnabled(false);
    //runButton->setEnabled(true);
		//resetButton->setEnabled(true);
		disconnectButton->setEnabled(true);

		/* Plot reset data */
		plot->clearData();
		/* Start camera */
		//camera->cameraStart("http://138.100.21.254:3010/jpg/1/image.jpg");
		//camera->cameraStart(networkTab->getIP(), networkTab->getCameraPort());
	}
	else
	{
		//camera->cameraStart("http://138.100.21.254:3010/jpg/1/image.jpg");
		printf("Not Connected\n");
	}
}

/*********************************
 *********************************/

void TabDialog::SocketRun ( void )
{
	resetButton->setEnabled(true);
	/* Send reset to socket */
	qtsecosocket->reset();
	//sleep(2);
	
	/* Plot reset data */
	plot->clearData();
	/* Send data to socket */
	qtsecosocket->run(networkTab, motorTab, controllerTab, perturbationTab, communicationTab );
	qtsecosocket->setDataEnd(false);
  //runButton->setEnabled(false);
}

/*********************************
 *********************************/

void TabDialog::SocketReset ( void )
{
	/* Plot reset data */
	plot->clearData();

	/* Send reset to socket */
	qtsecosocket->reset();
  //runButton->setEnabled(true);
}

/*********************************
 *********************************/

void TabDialog::SocketDisconnect ( void )
{
	qtsecosocket->disconnectFromServer();
	
	connectButton->setEnabled(true);
  //runButton->setEnabled(false);
	resetButton->setEnabled(false);
	disconnectButton->setEnabled(false);
	printf("Disconnected\n");

	camera->cameraStop();
}

/*********************************
 *********************************/

void TabDialog::configPlot ( void )
{
	/* Set Time limit */
	float timeLimit = communicationTab->getCommunicationTimeLimit(2);
	plot->setTimeLimit(timeLimit);

	/* Set variables to plot */
	for ( int i = 0 ; i < VARNAMES ; i++ )
	{
		plot->setCommVarFlag(i, communicationTab->getCommunicationParamValue (2, i));
	}
}

/*********************************
 *********************************/
/*********************************
 *********************************/

void TabDialog::correctQuit ( void )
{
	/* Send reset to socket */
	qtsecosocket->reset();
	SocketDisconnect();
}

void TabDialog::checkSocketStatus ( void )
{
	if ( !qtsecosocket->getSocketStatus() )
	{
	connectButton->setEnabled(true);
  //runButton->setEnabled(false);
  //resetButton->setEnabled(false);
  //disconnectButton->setEnabled(false);
	}
}

void TabDialog::checkDataEnd ( void )
{
	if ( qtsecosocket->getDataEnd() & qtsecosocket->getSocketStatus())
	{
    //runButton->setEnabled(true);
	}
}
